@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -83,7 +83,7 @@ gsf_input_bonobo_new (Bonobo_Stream const stream, GError **err)
 
 	if (stream == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"stream is NULL");
 		return NULL;
 	}
@@ -93,7 +93,7 @@ gsf_input_bonobo_new (Bonobo_Stream const stream, GError **err)
 	size = Bonobo_Stream_seek (stream, 0, Bonobo_Stream_SeekEnd, &ev);
 	if (BONOBO_EX (&ev)) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 					    "%s: %s",
 					    "Error seeking to get stream size",
 					    bonobo_exception_get_text (&ev));
@@ -103,7 +103,7 @@ gsf_input_bonobo_new (Bonobo_Stream const stream, GError **err)
 	Bonobo_Stream_seek (stream, 0, Bonobo_Stream_SeekSet, &ev);
 	if (BONOBO_EX (&ev)) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 					    "%s: %s",
 					    "Error seeking to get stream size",
 					    bonobo_exception_get_text (&ev));
@@ -115,7 +115,7 @@ gsf_input_bonobo_new (Bonobo_Stream const stream, GError **err)
 	info = Bonobo_Stream_getInfo (stream, 0, &ev);
 	if (BONOBO_EX (&ev)) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 					    "%s: %s",
 					    "Error getting stream info",
 					    bonobo_exception_get_text (&ev));
@@ -145,11 +145,9 @@ gsf_input_bonobo_finalize (GObject *obj)
 		g_object_unref (G_OBJECT (input->shared));
 	input->shared = NULL;
 
-	if (input->buf != NULL) {
-		g_free (input->buf);
-		input->buf  = NULL;
-		input->buf_size = 0;
-	}
+	g_free (input->buf);
+	input->buf = NULL;
+	input->buf_size = 0;
 
 	parent_class = g_type_class_peek (GSF_INPUT_TYPE);
 	if (parent_class && parent_class->finalize)
@@ -186,8 +184,7 @@ gsf_input_bonobo_read (GsfInput *input, size_t num_bytes,
 	if (buffer == NULL) {
 		if (binput->buf_size < num_bytes) {
 			binput->buf_size = num_bytes;
-			if (binput->buf != NULL)
-				g_free (binput->buf);
+			g_free (binput->buf);
 			binput->buf = g_new (guint8, binput->buf_size);
 		}
 		buffer = binput->buf;
