@@ -15,10 +15,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
+/* Lots of useful information in
+ *   http://www.aafassociation.org/html/specs/aafcontainerspec-v1.0.1.pdf
+ */
+
 #include <gsf-config.h>
 #include <gsf/gsf-infile-impl.h>
 #include <gsf/gsf-infile-msole.h>
@@ -100,12 +104,15 @@ static void ole_info_unref (MSOleInfo *info);
 
 /**
  * ole_get_block :
- * @ole    : the infile
- * @block  :
- * @buffer : optionally NULL
+ * @ole: the infile
+ * @block:
+ * @buffer: optionally NULL
  *
  * Read a block of data from the underlying input.
  * Be really anal.
+ *
+ * Returns: pointer to the buffer or NULL if there is an error or 0 bytes are
+ * requested.
  **/
 static guint8 const *
 ole_get_block (GsfInfileMSOle const *ole, guint32 block, guint8 *buffer)
@@ -124,15 +131,15 @@ ole_get_block (GsfInfileMSOle const *ole, guint32 block, guint8 *buffer)
 
 /**
  * ole_make_bat :
- * @metabat	: a meta bat to connect to the raw blocks (small or large)
- * @size_guess	: An optional guess as to how many blocks are in the file
- * @block	: The first block in the list.
- * @res		: where to store the result.
+ * @metabat: a meta bat to connect to the raw blocks (small or large)
+ * @size_guess: An optional guess as to how many blocks are in the file
+ * @block: The first block in the list.
+ * @res: where to store the result.
  *
  * Walk the linked list of the supplied block allocation table and build up a
  * table for the list starting in @block.
  *
- * Retrurns TRUE on error.
+ * Returns: TRUE on error.
  */
 static gboolean
 ole_make_bat (MSOleBAT const *metabat, size_t size_guess, guint32 block,
@@ -174,37 +181,48 @@ static void
 ols_bat_release (MSOleBAT *bat)
 {
 	if (bat->block != NULL) {
+		bat->num_blocks = 0;
 		g_free (bat->block);
 		bat->block = NULL;
-		bat->num_blocks = 0;
 	}
 }
 
 /**
  * ole_info_read_metabat :
- * @ole  :
- * @bats :
+ * @ole:
+ * @bats:
  *
  * A small utility routine to read a set of references to bat blocks
  * either from the OLE header, or a meta-bat block.
  *
- * Returns a pointer to the element after the last position filled.
+ * Returns: a pointer to the element after the last position filled.
  **/
 static guint32 *
-ole_info_read_metabat (GsfInfileMSOle *ole, guint32 *bats, guint32 max,
+ole_info_read_metabat (GsfInfileMSOle *ole, guint32 *bats, guint32 max_bat,
 		       guint32 const *metabat, guint32 const *metabat_end)
 {
 	guint8 const *bat, *end;
 
 	for (; metabat < metabat_end; metabat++) {
-		bat = ole_get_block (ole, *metabat, NULL);
-		if (bat == NULL)
-			return NULL;
-		end = bat + ole->info->bb.size;
-		for ( ; bat < end ; bat += BAT_INDEX_SIZE, bats++) {
-			*bats = GSF_LE_GET_GUINT32 (bat);
-			g_return_val_if_fail (*bats < max ||
-					      *bats >= BAT_MAGIC_METABAT, NULL);
+		if (*metabat != BAT_MAGIC_UNUSED) {
+			bat = ole_get_block (ole, *metabat, NULL);
+			if (bat == NULL)
+				return NULL;
+			end = bat + ole->info->bb.size;
+			for ( ; bat < end ; bat += BAT_INDEX_SIZE, bats++) {
+				*bats = GSF_LE_GET_GUINT32 (bat);
+				g_return_val_if_fail (*bats < max_bat ||
+						      *bats >= BAT_MAGIC_METABAT, NULL);
+			}
+		} else {
+			/* Looks like something in the wild sometimes creates
+			 * 'unused' entries in the metabat.  Let's assume that
+			 * corresponds to lots of unused blocks
+			 * http://bugzilla.gnome.org/show_bug.cgi?id=336858 */
+			unsigned i = ole->info->bb.size / BAT_INDEX_SIZE;
+			g_print ("Ping!\n");
+			while (i-- > 0)
+				bats[i] = BAT_MAGIC_UNUSED;
 		}
 	}
 	return bats;
@@ -212,9 +230,9 @@ ole_info_read_metabat (GsfInfileMSOle *ole, guint32 *bats, guint32 max,
 
 /**
  * gsf_ole_get_guint32s :
- * @dst :
- * @src :
- * @num_bytes :
+ * @dst:
+ * @src:
+ * @num_bytes:
  *
  * Copy some some raw data into an array of guint32.
  **/
@@ -271,11 +289,13 @@ ole_dirent_cmp (MSOleDirent const *a, MSOleDirent const *b)
 
 /**
  * ole_dirent_new :
- * @ole    :
- * @entry  :
- * @parent : optional
+ * @ole:
+ * @entry:
+ * @parent: optional
  *
  * Parse dirent number @entry and recursively handle its siblings and children.
+ *
+ * Returns: The dirent
  **/
 static MSOleDirent *
 ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent)
@@ -304,6 +324,11 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent)
 		g_warning ("Unknown stream type 0x%x", type);
 		return NULL;
 	}
+	if (!parent && type != DIRENT_TYPE_ROOTDIR) {
+		/* See bug 346118.  */
+		g_warning ("Root directory is not marked as such.");
+		type = DIRENT_TYPE_ROOTDIR;
+	}
 
 	/* It looks like directory (and root directory) sizes are sometimes bogus */
 	size = GSF_LE_GET_GUINT32 (data + DIRENT_FILE_SIZE);
@@ -423,13 +448,13 @@ ole_info_ref (MSOleInfo *info)
 }
 
 /**
- * ole_dup :
- * @src :
+ * ole_dup:
+ * @src:
  *
  * Utility routine to _partially_ replicate a file.  It does NOT copy the bat
  * blocks, or init the dirent.
  *
- * Return value: the partial duplicate.
+ * Returns: the partial duplicate.
  **/
 static GsfInfileMSOle *
 ole_dup (GsfInfileMSOle const *src, GError **err)
@@ -442,7 +467,7 @@ ole_dup (GsfInfileMSOle const *src, GError **err)
 	input = gsf_input_dup (src->input, err);
 	if (input == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 					    "Failed to duplicate input stream");
 		return NULL;
 	}
@@ -458,13 +483,13 @@ ole_dup (GsfInfileMSOle const *src, GError **err)
 
 /**
  * ole_init_info :
- * @ole :
- * @err : optionally NULL
+ * @ole:
+ * @err: optionally NULL
  *
  * Read an OLE header and do some sanity checking
  * along the way.
  *
- * Return value: TRUE on error setting @err if it is supplied.
+ * Returns: TRUE on error setting @err if it is supplied.
  **/
 static gboolean
 ole_init_info (GsfInfileMSOle *ole, GError **err)
@@ -482,7 +507,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	    NULL == (header = gsf_input_read (ole->input, OLE_HEADER_SIZE, NULL)) ||
 	    0 != memcmp (header, signature, sizeof (signature))) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"No OLE2 signature");
 		return TRUE;
 	}
@@ -499,9 +524,10 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	 * 2) It makes no sense to have a block larger than 2^31 for now.
 	 *    Maybe relax this later, but not much.
 	 */
-	if (6 > bb_shift || bb_shift >= 31 || sb_shift > bb_shift) {
+	if (6 > bb_shift || bb_shift >= 31 || sb_shift > bb_shift ||
+	    (gsf_input_size (ole->input) >> bb_shift) < 1) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Unreasonable block sizes");
 		return TRUE;
 	}
@@ -522,8 +548,10 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	info->max_block	     = (gsf_input_size (ole->input) - OLE_HEADER_SIZE) / info->bb.size;
 	info->sb_file	     = NULL;
 
-	if (info->num_sbat == 0 && info->sbat_start != BAT_MAGIC_END_OF_CHAIN) {
-		g_warning ("There is are not supposed to be any blocks in the small block allocation table, yet there is a link to some.  Ignoring it.");
+	if (info->num_sbat == 0 &&
+	    info->sbat_start != BAT_MAGIC_END_OF_CHAIN &&
+	    info->sbat_start != BAT_MAGIC_UNUSED) {
+		g_warning ("There are not supposed to be any blocks in the small block allocation table, yet there is a link to some.  Ignoring it.");
 	}
 
 	/* very rough heuristic, just in case */
@@ -567,6 +595,13 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 			last = num_bat;
 		} else if (num_metabat > 0) {
 			metabat_block = metabat[last];
+			if (num_bat < last) {
+				/* ::num_bat and ::num_metabat are
+				 * inconsistent.  There are too many metabats
+				 * for the bat count in the header. */
+				ptr = NULL;
+				break;
+			}
 			num_bat -= last;
 		}
 
@@ -576,7 +611,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 
 	if (ptr == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Inconsistent block allocation table");
 		return TRUE;
 	}
@@ -584,7 +619,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	/* Read the directory's bat, we do not know the size */
 	if (ole_make_bat (&info->bb.bat, 0, dirent_start, &ole->bat)) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Problems making block allocation table");
 		return TRUE;
 	}
@@ -593,7 +628,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	ole->dirent = info->root_dir = ole_dirent_new (ole, 0, NULL);
 	if (ole->dirent == NULL) {
 		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
+			*err = g_error_new (gsf_input_error_id (), 0,
 				"Problems reading directory");
 		return TRUE;
 	}
@@ -685,8 +720,7 @@ gsf_infile_msole_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 	/* damn, we need to copy it block by block */
 	if (buffer == NULL) {
 		if (ole->stream.buf_size < num_bytes) {
-			if (ole->stream.buf != NULL)
-				g_free (ole->stream.buf);
+			g_free (ole->stream.buf);
 			ole->stream.buf_size = num_bytes;
 			ole->stream.buf = g_new (guint8, num_bytes);
 		}
@@ -762,7 +796,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 		sb_file = ole_info_get_sb_file (parent);
 		if (!sb_file) {
 			if (err != NULL)
-				*err = g_error_new (gsf_input_error (), 0,
+				*err = g_error_new (gsf_input_error_id (), 0,
 						    "Failed to access child");
 			g_object_unref (G_OBJECT (child));
 			return NULL;
@@ -891,13 +925,13 @@ GSF_CLASS (GsfInfileMSOle, gsf_infile_msole,
 
 /**
  * gsf_infile_msole_new :
- * @source :
- * @err   :
+ * @source:
+ * @err:
  *
  * Opens the root directory of an MS OLE file.
- * NOTE : adds a reference to @source
+ * <note>This adds a reference to @source.</note>
  *
- * Returns : the new ole file handler
+ * Returns: the new ole file handler
  **/
 GsfInfile *
 gsf_infile_msole_new (GsfInput *source, GError **err)
@@ -932,7 +966,7 @@ gsf_infile_msole_new (GsfInput *source, GError **err)
  * Retrieves the 16 byte indentifier (often a GUID in MS Windows apps)
  * stored within the directory associated with @ole and stores it in @res.
  *
- * Returns TRUE on success
+ * Returns: TRUE on success
  **/
 gboolean
 gsf_infile_msole_get_class_id (GsfInfileMSOle const *ole, guint8 *res)
