@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -185,6 +185,7 @@ gsf_output_csv_init (GObject *obj)
 	GsfOutputCsv *csv = (GsfOutputCsv *)obj;
 	csv->quoting_triggers = g_strdup ("");
 	csv->eol = g_strdup ("\n");
+	csv->eol_len = strlen (csv->eol);
 	csv->buf = g_string_new (NULL);
 }
 
