@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -23,6 +23,7 @@
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
 #include <gobject/gvaluecollector.h>
+#include <glib/gi18n-lib.h>
 
 #include <ctype.h>
 #include <stdio.h>
@@ -42,6 +43,11 @@
 
 static void base64_init (void);
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
+#endif
+
 /**
  * gsf_init :
  *
@@ -50,6 +56,20 @@ static void base64_init (void);
 void
 gsf_init (void)
 {
+#ifdef ENABLE_NLS
+#ifdef G_OS_WIN32
+#undef GNOMELOCALEDIR
+	gchar *prefix = g_win32_get_package_installation_directory (NULL, dll_name);
+	gchar *GNOMELOCALEDIR = g_build_filename (prefix, "lib/locale", NULL);
+	g_free (prefix);
+#endif
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+#ifdef G_OS_WIN32
+	g_free (GNOMELOCALEDIR);
+#endif
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+#endif
+
 	g_type_init ();
 	base64_init ();
 }
@@ -58,22 +78,32 @@ gsf_init (void)
  * gsf_shutdown:
  * 
  * De-intializes the GSF library
- */
+ **/
 void
 gsf_shutdown (void)
 {
 }
 
-static void
-gsf_mem_dump_full (guint8 const *ptr, size_t len, guint offset)
+void
+gsf_init_dynamic (G_GNUC_UNUSED GTypeModule *module)
 {
-	size_t i, j, off;
+}
+
+void
+gsf_shutdown_dynamic (G_GNUC_UNUSED GTypeModule *module)
+{
+}
+
+static void
+gsf_mem_dump_full (guint8 const *ptr, size_t len, gsf_off_t offset)
+{
+	gsf_off_t i, j, off;
 
 	for (i = 0 ; i < (len+15)/16 ; i++) {
-		g_print ("%8x | ", i*16 + offset);
+		g_print ("%8lx | ", (long)(i*16 + offset));
 		for (j = 0;j < 16; j++) {
 			off = j + (i << 4);
-			off<len ? g_print("%2x ", ptr[off]) : g_print("XX ");
+			off<len ? g_print("%02x ", ptr[off]) : g_print("XX ");
 		}
 		g_print ("| ");
 		for (j = 0 ; j < 16 ; j++) {
@@ -107,7 +137,8 @@ gsf_mem_dump (guint8 const *ptr, size_t len)
 void
 gsf_input_dump (GsfInput *input, gboolean dump_as_hex)
 {
-	size_t size, count, offset = 0;
+	gsf_off_t offset = 0;
+	size_t size, count;
 	guint8 const *data;
 
 	/* read in small blocks to excercise things */
@@ -129,6 +160,15 @@ gsf_input_dump (GsfInput *input, gboolean dump_as_hex)
 		fflush (stdout);
 }
 
+/**
+ * gsf_le_get_guint64
+ * @p: pointer to storage
+ *
+ * Interpret binary data as a guint64 (8 byte unsigned integer type) in little
+ * endian order.
+ *
+ * Returns: interpreted data
+ */
 guint64
 gsf_le_get_guint64 (void const *p)
 {
@@ -164,6 +204,15 @@ gsf_le_get_guint64 (void const *p)
 #endif
 }
 
+/**
+ * gsf_le_get_float :
+ * @p: pointer to storage
+ *
+ * Interpret binary data as a float in little endian order.
+ *
+ *
+ * Returns: interpreted data
+ */
 float
 gsf_le_get_float (void const *p)
 {
@@ -199,15 +248,22 @@ gsf_le_get_float (void const *p)
 #endif
 }
 
+/**
+ * gsf_le_set_float :
+ * @p: pointer to storage
+ * @f: float to be stored
+ *
+ * Store a value of type float in memory in little endian order.
+ */
 void
-gsf_le_set_float (void *p, float d)
+gsf_le_set_float (void *p, float f)
 {
 #if G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (float) == 4) {
 		int     i;
-		guint8 *t  = (guint8 *)&d;
+		guint8 *t  = (guint8 *)&f;
 		guint8 *p2 = (guint8 *)p;
-		int     sd = sizeof (d);
+		int     sd = sizeof (f);
 
 		for (i = 0; i < sd; i++)
 			p2[sd - 1 - i] = t[i];
@@ -220,7 +276,7 @@ gsf_le_set_float (void *p, float d)
 		 * On i86, we could access directly, but Alphas require
 		 * aligned access.
 		 */
-		memcpy (p, &d, sizeof (d));
+		memcpy (p, &f, sizeof (f));
 	} else {
 		g_error ("Little endian machine, but weird size of floats");
 	}
@@ -229,6 +285,14 @@ gsf_le_set_float (void *p, float d)
 #endif
 }
 
+/**
+ * gsf_le_get_double :
+ * @p: pointer to storage
+ *
+ * Interpret binary data as a double in little endian order.
+ *
+ * Returns: interpreted data
+ */
 double
 gsf_le_get_double (void const *p)
 {
@@ -269,6 +333,13 @@ gsf_le_get_double (void const *p)
 #endif
 }
 
+/**
+ * gsf_le_set_double :
+ * @p: pointer to storage
+ * @d: double to be stored
+ *
+ * Store a value of type double in memory in little endian order
+ */
 void
 gsf_le_set_double (void *p, double d)
 {
@@ -333,8 +404,10 @@ gsf_extension_pointer (char const *path)
 }
 
 /**
- * gsf_iconv_close : A utility wrapper to safely close an iconv handle
- * @handle :
+ * gsf_iconv_close :
+ * @handle : handle to be closed.
+ *
+ * A utility wrapper to safely close an iconv handle.
  **/
 void
 gsf_iconv_close (GIConv handle)
@@ -389,8 +462,8 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 /* dont touch this file without my permission - Michael */
@@ -415,8 +488,20 @@ base64_init(void)
 	camel_mime_base64_rank['='] = 0;
 }
 
-/* call this when finished encoding everything, to
-   flush off the last little bit */
+/**
+ * gsf_base64_encode_close :
+ * @in : Data to be encoded
+ * @inlen : Length of data to be encoded
+ * @break_lines : Whether to use line breaks
+ * @out : Encoded data.
+ * @state: holds the number of bits that are stored in @save
+ * @save: leftover bits that have not yet been decoded
+ *
+ * This funcion should be called to when finished encoding everything, to
+ * flush off the last little bit.
+ *
+ * Returns :
+ */
 size_t
 gsf_base64_encode_close (guint8 const *in, size_t inlen,
 			 gboolean break_lines, guint8 *out, int *state, unsigned int *save)
@@ -459,11 +544,21 @@ gsf_base64_encode_close (guint8 const *in, size_t inlen,
 	return outptr-out;
 }
 
-/*
-  performs an 'encode step', only encodes blocks of 3 characters to the
-  output at a time, saves left-over state in state and save (initialise to
-  0 on first invocation).
-*/
+/**
+ * gsf_base64_encode_step :
+ * @in : input stream
+ * @len : max length of data to decode
+ * @break_lines : Whether to use line breaks
+ * @out : output stream
+ * @state : holds the number of bits that are stored in @save
+ * @save : leftover bits that have not yet been decoded
+ *
+ * Performs an 'encode step', only encodes blocks of 3 characters from @in into
+ * the output @out at a time, saves left-over state in @state and @save
+ * (initialise to 0 on first invocation).
+ *
+ * Returns : the number of bytes encoded
+ */
 size_t
 gsf_base64_encode_step (guint8 const *in, size_t len,
 			gboolean break_lines, guint8 *out, int *state, unsigned int *save)
@@ -604,6 +699,15 @@ gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
 	return outptr-out;
 }
 
+/**
+ * gsf_base64_encode_simple :
+ * @data : data stream
+ * @len : max length of data to encode
+ *
+ * Encodes data from @data back into @data using base64 encoding.
+ *
+ * Returns : the number of bytes encoded
+ */
 guint8 *
 gsf_base64_encode_simple (guint8 const *data, size_t len)
 {
@@ -622,6 +726,15 @@ gsf_base64_encode_simple (guint8 const *data, size_t len)
 	return out;
 }
 
+/**
+ * gsf_base64_decode_simple :
+ * @data : data stream
+ * @len : max length of data to decode
+ *
+ * Decodes a chunk of base64 encoded data from @data back into @data.
+ *
+ * Returns : the number of bytes converted
+ */
 size_t
 gsf_base64_decode_simple (guint8 *data, size_t len)
 {
@@ -725,3 +838,26 @@ gsf_property_settings_free (GParameter *params,
 		g_value_unset (&params[n_params].value);
 	g_free (params);
 }
+
+
+
+/* Errors */
+
+/**
+ * gsf_error_quark:
+ *
+ * Returns the #GQuark used to identify libgsf errors in #GError structures.
+ * Specific error codes come from the #GsfError enumeration.
+ * 
+ * Return value: A #GQuark.
+ **/
+GQuark
+gsf_error_quark (void)
+{
+	static GQuark quark;
+
+	if (quark == 0)
+		quark = g_quark_from_static_string ("gsf-error-quark");
+
+	return quark;
+}
