@@ -2,7 +2,7 @@
 /*
  * test-textline.c:
  *
- * Copyright (C) 2002-2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -30,7 +30,7 @@ test (int argc, char *argv[])
 {
 	GsfInput     	 *input;
 	GsfInputTextline *textline;
-	GError       	 *err;
+	GError       	 *err = NULL;
 	unsigned char *line;
 
 	if (argc < 2) {
@@ -39,7 +39,7 @@ test (int argc, char *argv[])
 	}
 
 	fprintf (stderr, "%s\n", argv[1]);
-	input = GSF_INPUT (gsf_input_stdio_new (argv[1], &err));
+	input = gsf_input_stdio_new (argv[1], &err);
 	if (input == NULL) {
 
 		g_return_val_if_fail (err != NULL, 1);
@@ -48,7 +48,7 @@ test (int argc, char *argv[])
 		g_error_free (err);
 		return 1;
 	}
-	textline = gsf_input_textline_new (input);
+	textline = (GsfInputTextline *)gsf_input_textline_new (input);
 	if (textline == NULL) {
 		g_warning ("unable to create a textline");
 		return 2;
